@@ -420,11 +420,7 @@ agg_i2recv(iph1, msg)
 	natt_select_type(iph1);
 	
 	/* payload existency check */
-	if (iph1->dhpub_p == NULL || iph1->nonce_p == NULL) {
-		plog(LLV_ERROR, LOCATION, iph1->remote,
-			"required payloads missing from isakmp message.\n");
-		goto end;
-	}
+	/* XXX to be checked each authentication method. */
 
 	/* verify identifier */
 	if (ipsecdoi_checkid1(iph1) != 0) {
@@ -457,7 +453,18 @@ agg_i2recv(iph1, msg)
 	     pa->type != ISAKMP_NPTYPE_NONE;
 	     pa++)
 	{
-		if (pa->type == iph1->natd_payload_type) {
+		if (pa->type == ISAKMP_NPTYPE_NATD_RFC ||
+			pa->type == ISAKMP_NPTYPE_NATD_DRAFT ||
+			pa->type == ISAKMP_NPTYPE_NATD_BADDRAFT)
+		{
+			if (pa->type != iph1->natd_payload_type) {
+				plog(LLV_ERROR, LOCATION, iph1->remote,
+					"ignore the packet, "
+					"received unexpected natd payload type %d.\n",
+					pa->type);
+				goto end;
+			}			
+			
 			natd_match_t match = natd_matches(iph1, pa->ptr);
 			iph1->natt_flags |= natt_natd_received;
 			if ((match & natd_match_local) != 0)
@@ -698,17 +705,10 @@ agg_i2send(iph1, msg)
 
 #ifdef IKE_NAT_T
 	if (natd_type) {
-		if ((iph1->natt_flags & NATT_TYPE_MASK) == natt_type_apple) {
-			if (iph1->local_natd)
-				p = set_isakmp_payload(p, iph1->local_natd, natd_type);
-			if (iph1->remote_natd)
-				p = set_isakmp_payload(p, iph1->remote_natd, ISAKMP_NPTYPE_NONE);
-		} else {
-			if (iph1->remote_natd)
-				p = set_isakmp_payload(p, iph1->remote_natd, natd_type);
-			if (iph1->local_natd)
-				p = set_isakmp_payload(p, iph1->local_natd, ISAKMP_NPTYPE_NONE);
-		}
+		if (iph1->local_natd)
+			p = set_isakmp_payload(p, iph1->local_natd, natd_type);
+		if (iph1->remote_natd)
+			p = set_isakmp_payload(p, iph1->remote_natd, ISAKMP_NPTYPE_NONE);
 	}
 #endif
 
@@ -847,11 +847,7 @@ agg_r1recv(iph1, msg)
 	}
 
 	/* payload existency check */
-	if (iph1->dhpub_p == NULL || iph1->nonce_p == NULL) {
-		plog(LLV_ERROR, LOCATION, iph1->remote,
-			"required payloads missing from isakmp message.\n");
-		goto end;
-	}
+	/* XXX to be checked each authentication method. */
 
 	/* verify identifier */
 	if (ipsecdoi_checkid1(iph1) != 0) {
@@ -1159,17 +1155,10 @@ agg_r1send(iph1, msg)
 #ifdef IKE_NAT_T
 		if (nattvid) {
 			p = set_isakmp_payload(p, nattvid, iph1->natd_payload_type);
-			if ((iph1->natt_flags & NATT_TYPE_MASK) == natt_type_apple) {
-				if (iph1->local_natd)
-					p = set_isakmp_payload(p, iph1->local_natd, iph1->natd_payload_type);
-				if (iph1->remote_natd)
-					p = set_isakmp_payload(p, iph1->remote_natd, ISAKMP_NPTYPE_NONE);
-			} else {
-				if (iph1->remote_natd)
-					p = set_isakmp_payload(p, iph1->remote_natd, iph1->natd_payload_type);
-				if (iph1->local_natd)
-					p = set_isakmp_payload(p, iph1->local_natd, ISAKMP_NPTYPE_NONE);
-			}
+			if (iph1->local_natd)
+				p = set_isakmp_payload(p, iph1->local_natd, iph1->natd_payload_type);
+			if (iph1->remote_natd)
+				p = set_isakmp_payload(p, iph1->remote_natd, ISAKMP_NPTYPE_NONE);
 		}
 #endif
 		break;
@@ -1253,17 +1242,10 @@ agg_r1send(iph1, msg)
 #ifdef IKE_NAT_T
 	if (nattvid) {
 		p = set_isakmp_payload(p, nattvid, iph1->natd_payload_type);
-		if ((iph1->natt_flags & NATT_TYPE_MASK) == natt_type_apple) {
-			if (iph1->local_natd)
-				p = set_isakmp_payload(p, iph1->local_natd, iph1->natd_payload_type);
-			if (iph1->remote_natd)
-				p = set_isakmp_payload(p, iph1->remote_natd, ISAKMP_NPTYPE_NONE);
-		} else {
-			if (iph1->remote_natd)
-				p = set_isakmp_payload(p, iph1->remote_natd, iph1->natd_payload_type);
-			if (iph1->local_natd)
-				p = set_isakmp_payload(p, iph1->local_natd, ISAKMP_NPTYPE_NONE);
-		}		
+		if (iph1->local_natd)
+			p = set_isakmp_payload(p, iph1->local_natd, iph1->natd_payload_type);
+		if (iph1->remote_natd)
+			p = set_isakmp_payload(p, iph1->remote_natd, ISAKMP_NPTYPE_NONE);
 	}
 #endif
 
@@ -1377,7 +1359,15 @@ agg_r2recv(iph1, msg0)
 		case ISAKMP_NPTYPE_NATD_DRAFT:
 		case ISAKMP_NPTYPE_NATD_BADDRAFT:
 #ifdef IKE_NAT_T
-			if (pa->type == iph1->natd_payload_type) {
+			if (pa->type != iph1->natd_payload_type) {
+				plog(LLV_ERROR, LOCATION, iph1->remote,
+					"ignore the packet, "
+					"received unexpected natd payload type %d.\n",
+					pa->type);
+				goto end;
+			}
+				
+			{
 				natd_match_t match = natd_matches(iph1, pa->ptr);
 				iph1->natt_flags |= natt_natd_received;
 				if ((match & natd_match_local) != 0)
