@@ -6,18 +6,18 @@
 #ifndef LOCKD_LOCK_H
 #define LOCKD_LOCK_H
 struct nlm4_holder * testlock(struct nlm4_lock *lock, bool_t exclusive, int flags);
-enum nlm_stats getlock(nlm4_lockargs *lckarg, struct svc_req *rqstp, const int flags);
-enum nlm_stats unlock(nlm4_lock *lock, const int flags);
-enum nlm_stats cancellock(nlm4_cancargs *args, const int flags);
+enum nlm4_stats getlock(nlm4_lockargs *lckarg, struct svc_req *rqstp, const int flags);
+enum nlm4_stats unlock(nlm4_lock *lock, const int flags);
+enum nlm4_stats cancellock(nlm4_cancargs *args, const int flags);
 int lock_answer(int version, netobj *netcookie, nlm4_lock *lock, int flags, int result);
-enum nlm_stats getshare(nlm_shareargs *shrarg, struct svc_req *rqstp, const int flags);
-enum nlm_stats unshare(nlm_shareargs *shrarg, struct svc_req *rqstp);
+enum nlm4_stats getshare(nlm_shareargs *shrarg, struct svc_req *rqstp, const int flags);
+enum nlm4_stats unshare(nlm_shareargs *shrarg, struct svc_req *rqstp, const int flags);
 void do_free_all(const char *hostname);
 void granted_failed(nlm4_res *arg);
 
 void notify(const char *hostname, const int state);
 
-void monitor_lock_host_by_name(const char *hostname);
+void monitor_lock_host_by_name(const char *, const struct sockaddr *);
 void monitor_lock_host_by_addr(const struct sockaddr *addr);
 void unmonitor_lock_host(const char *hostname);
 
